<?php
/**
 * @package	PayAnyWay_for_RS_Membership
 * @copyright	(c) 2017 PayAnyWay
 * @link		https://www.payanyway.ru
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

defined('_JEXEC') or die('Restricted access');

jimport('joomla.plugin.plugin');

class plgSystemRSMembershipPayanyway extends JPlugin
{
	public function __construct($subject, $config)
	{
		parent::__construct($subject, $config);

		if ($this->canRun() && $this->params)
		{
			require_once JPATH_ADMINISTRATOR . '/components/com_rsmembership/helpers/rsmembership.php';
			$this->_loadLanguage();
			$name = $this->params->get('payment_name', 'PayAnyWay');
			RSMembership::addPlugin($this->getTranslation($name), 'rsmembershippayanyway');
		}
	}

	protected function canRun()
	{
		return file_exists(JPATH_ADMINISTRATOR . '/components/com_rsmembership/helpers/rsmembership.php');
	}

	protected function getTranslation($text)
	{
		$lang = JFactory::getLanguage();
		$key = str_replace(' ', '_', $text);
		if ($lang->hasKey($key))
		{
			return JText::_($key);
		} else
		{
			return $text;
		}
	}
	
	public function onAfterDispatch()
    {
        $app = JFactory::getApplication();
		$payanyway = $app->input->get('payanyway');

        if (!empty($payanyway)) {
			$this->onPaymentNotification($app);
			
        }
    }
	
	const paw_kassa_VAT0     = '1104';  // НДС 0%
	const paw_kassa_VAT10    = '1103';  // НДС 10%
	const paw_kassa_VAT18    = '1102';  // НДС 18%
	const paw_kassa_VATNOVAT = '1105';  // НДС не облагается
	const paw_kassa_VATWR10  = '1107';  // НДС с рассч. ставкой 10%
	const paw_kassa_VATWR18  = '1106';  // НДС с рассч. ставкой 18%

	private static function monetaPayURLResponse($mnt_id, $mnt_transaction_id, $mnt_data_integrity_code, $success = false,
												 $repeatRequest = false, $echo = true, $die = true, $kassa_inventory = null, $kassa_customer = null, $kassa_delivery = null)
	{
		if ($success === true)
			$resultCode = '200';
		elseif ($repeatRequest === true)
			$resultCode = '402';
		else
			$resultCode = '500';
		$mnt_signature = md5($resultCode.$mnt_id.$mnt_transaction_id.$mnt_data_integrity_code);
		$response = '<?xml version="1.0" encoding="UTF-8" ?>'."\n";
		$response .= "<MNT_RESPONSE>\n";
		$response .= "<MNT_ID>{$mnt_id}</MNT_ID>\n";
		$response .= "<MNT_TRANSACTION_ID>{$mnt_transaction_id}</MNT_TRANSACTION_ID>\n";
		$response .= "<MNT_RESULT_CODE>{$resultCode}</MNT_RESULT_CODE>\n";
		$response .= "<MNT_SIGNATURE>{$mnt_signature}</MNT_SIGNATURE>\n";
		if (!empty($kassa_inventory) || !empty($kassa_customer) || !empty($kassa_delivery))
		{
			$response .= "<MNT_ATTRIBUTES>\n";
			foreach (array('INVENTORY' => $kassa_inventory, 'CUSTOMER' => $kassa_customer, 'DELIVERY' => $kassa_delivery) as $k => $v)
				if (!empty($v))
					$response .= "<ATTRIBUTE><KEY>{$k}</KEY><VALUE>{$v}</VALUE></ATTRIBUTE>\n";
			$response .= "</MNT_ATTRIBUTES>\n";
		}
		$response .= "</MNT_RESPONSE>\n";
		if ($echo === true)
		{
			header("Content-type: application/xml");
			echo $response;
		}
		else
			return $response;
		if ($die === true)
			die;
		return '';
	}

	protected function onPaymentNotification()
	{
		if (!$this->canRun()) return;
		$name = $this->params->get('payment_name', 'PayAnyWay');

		ob_end_clean();

		$jinput = JFactory::getApplication()->input;
		$mnt_id = $jinput->get('MNT_ID', '', 'string');
		$mnt_amount = $jinput->get('MNT_AMOUNT', '', 'string');
		$mnt_currency_code = $jinput->get('MNT_CURRENCY_CODE', '', 'string');
		$mnt_subscriber_id = $jinput->get('MNT_SUBSCRIBER_ID', '', 'string');
		$mnt_test_mode = $jinput->get('MNT_TEST_MODE', '', 'string');
		$mnt_signature = $jinput->get('MNT_SIGNATURE', '', 'string');
		$mnt_operation_id = $jinput->get('MNT_OPERATION_ID', '', 'string');
		$mnt_transaction_id = $jinput->get('MNT_TRANSACTION_ID', '', 'string');
		$mnt_data_integrity_code = $this->params->get('mnt_dataintegrity_code');
		$signature = md5($mnt_id.$mnt_transaction_id.$mnt_operation_id.$mnt_amount.$mnt_currency_code.$mnt_subscriber_id.$mnt_test_mode.$mnt_data_integrity_code);
		if ($signature == $mnt_signature)
		{
			$transaction = $this->getTransaction($mnt_transaction_id);
			if ($transaction)
			{
				$userEmail = $transaction->user_email;
				$membershipId = intval(preg_replace('/.*membership_id=(\d+).*/', '$1', $transaction->params));
				$membership = $this->getMembership($membershipId);
				if ($membership)
				{
					$membershipName = $membership->name;
					$inventoryPositions = [
						'name' => trim(preg_replace("/&?[a-z0-9]+;/i", "", htmlspecialchars($membershipName))),
						'price' => $mnt_amount,
						'quantity' => '1',
						'vatTag' => self::paw_kassa_VATNOVAT
					];
					$kassa_inventory = json_encode($inventoryPositions);

					JTable::addIncludePath(JPATH_ADMINISTRATOR.'/components/com_rsmembership/tables');
					$row    = JTable::getInstance('Transaction','RSMembershipTable');
					$date   = JFactory::getDate();
					$row->id		 = $transaction->id;
					$row->user_id    = $transaction->user_id;
					$row->user_email = $transaction->user_email;
					$row->user_data  = $transaction->user_data;
					$row->type       = $transaction->type;
					$row->params     = $transaction->params;
					$row->date       = $date->toSql();
					$row->ip         = $transaction->ip;
					$row->price      = $transaction->price;
					$row->coupon     = $transaction->coupon;
					$row->currency   = $transaction->currency;
					$row->hash       = $transaction->hash;
					$row->custom     = $transaction->custom;
					$row->gateway    = $name;
					$row->status     = 'completed';
					$row->response_log = $transaction->response_log;
					// store the transaction
					$row->store();

                    $paymentPlugin = 'PayAnyWay';
                    $paymentPlugin == 'none' ? 'No Gateway' : RSMembership::getPlugin($paymentPlugin);

                    $extras = [];

                    // Trigger the payment plugin
                    $delay = false;
                    $args  = array(
                        'plugin' 		=> $paymentPlugin,
                        'data' 			=> &$this->data,
                        'extras'		=> $extras,
                        'membership' 	=> $membership,
                        'transaction' 	=> &$row,
                        'html' 			=> &$this->html
                    );

                    $app	 = JFactory::getApplication();
                    $returns = $app->triggerEvent('onMembershipPayment', $args);

                    // Finalize the transaction (send emails)
                    RSMembership::finalize($row->id);

                    // Approve the transaction
                    if ($row->status == 'completed' || (!$this->showPaymentOptions() && $membership->activation != MEMBERSHIP_ACTIVATION_MANUAL) || $membership->activation == MEMBERSHIP_ACTIVATION_INSTANT) {
                        RSMembership::approve($row->id, true);
                    }

					self::monetaPayURLResponse($mnt_id, $mnt_transaction_id, $mnt_data_integrity_code, true, false, true, true, $kassa_inventory , $userEmail);
				}
			}
		}
		
		echo 'FAIL';
		die;
	}

	protected function getTransaction($value, $column = 'custom') {
		$db     = JFactory::getDbo();
		$query  = $db->getQuery(true);

		$query->select('*')
			->from($db->qn('#__rsmembership_transactions'))
			->where($db->qn($column).' = '.$db->q($value));
		$db->setQuery($query);
		return $db->loadObject();
	}

	protected function getMembership($value, $column = 'id') {
		$db     = JFactory::getDbo();
		$query  = $db->getQuery(true);

		$query->select('*')
			->from($db->qn('#__rsmembership_memberships'))
			->where($db->qn($column).' = '.$db->q($value));
		$db->setQuery($query);
		return $db->loadObject();
	}

	public function onMembershipPayment($plugin, $data, $extra, $membership, &$transaction, $html)
	{
		if(!$this->canRun())
			return false;
		if ($plugin != 'rsmembershippayanyway')
			return false;

		$transaction->custom = uniqid(strval($transaction->user_id), true);
		$params = $this->params;
		$mnt_id = $params->get('mnt_id');
		$mnt_amount = $transaction->price;
		$mnt_transaction_id = $transaction->custom;
		$mnt_test_mode = ($params->get('mnt_test_mode') == "ON") ? '1' : '0';
		$av_curr = ['RUB', 'EUR', 'USD'];
		$mnt_currency_code = in_array($transaction->currency, $av_curr) ?
			$transaction->currency : (in_array($params->get('in_curr'), $av_curr) ? $params->get('in_curr') : 'RUB');
		$mnt_data_integrity_code = $params->get('mnt_dataintegrity_code');
		$mnt_signature = md5($mnt_id.$mnt_transaction_id.$mnt_amount.$mnt_currency_code.$mnt_test_mode.$mnt_data_integrity_code);
		$mnt_server = $params->get('payment_action');

		// $html = '';
		$html .= "<form action='{$mnt_server}' method='post' name='pawform'>";
        $html .="<input type='hidden' name='MNT_ID' value='{$mnt_id}'>";
		$html .="<input type='hidden' name='MNT_AMOUNT' value='{$mnt_amount}'>";
        $html .="<input type='hidden' name='MNT_TRANSACTION_ID' value='{$mnt_transaction_id}'>";
        $html .="<input type='hidden' name='MNT_TEST_MODE' value='{$mnt_test_mode}'>";
		$html .="<input type='hidden' name='MNT_SIGNATURE' value='{$mnt_signature}'>";
        $html .="<input type='hidden' name='MNT_DESCRIPTION' value='{$membership->name} ({$data->email})'>";
        $html .="<input type='hidden' name='MNT_CURRENCY_CODE' value='{$mnt_currency_code}'>";
		$html .="<div class='form-actions'><input type='submit' class='button btn btn-success pull-right' value='".
			JText::_('COM_RSMEMBERSHIP_PAYANYWAY_PAY')."'></div></form>";
		return $html;
	}


	protected function _loadLanguage()
	{
		$this->loadLanguage('plg_system_rsmembershippayanyway', JPATH_ADMINISTRATOR);
	}

}
